
/*
 *  wc.c
 *  Copyright (C) 1999 Brad Proctor
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 * $Id: wc.c,v 1.4 1999/06/21 15:04:55 brad Exp $
 */

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include "version.h"

#define  BUF_SIZE    65536

static char help_text[] =
{
   "Usage: wc [-c|-m][-lw][file...]\n\n"
   "  -c         Print the byte count\n"
   "  -l         Print the line count\n"
   "  -m         Print the character count\n"
   "  -w         Print the word count\n"
   "  --help     Print this help message\n"
   "  --version  Print the program version\n\n"
   "Report bugs to fredlie@sprynet.com"
};

static unsigned long bytes_count = 0;
static unsigned long words_count = 0;
static unsigned long lines_count = 0;

static char buf[BUF_SIZE];
static char flag_bytes = 0;
static char flag_chars = 0;
static char flag_words = 0;
static char flag_lines = 0;
static FILE *in = stdin;

static void do_count(void)
{
   unsigned int i, word_on = 1;
   char *ptr;

   if(!flag_words)
   {
      if(flag_bytes && !flag_lines)
      {
         fseek(in, 0L, SEEK_END);
         bytes_count = ftell(in);
         return;
      }
      while((i = fread(buf, 1, BUF_SIZE, in)) > 0)
      {
         bytes_count += i;
         ptr = buf;
         while(i--)
         {
            if(*ptr == '\n')
               lines_count++;
            ++ptr;
         }
      }
      return;
   }
   while((i = fread(buf, 1, BUF_SIZE, in)) > 0)
   {
      bytes_count += i;
      ptr = buf;
      while(i--)
      {
         if(isspace(*ptr))
         {
            if(word_on)
               words_count++;
            word_on = 0;
            if(*ptr == '\n')
               lines_count++;
         }
         else
            word_on = 1;
         ++ptr;
      }
   }
}

/*
 * Main
 */

int main(int argc, char **argv)
{
   int i;
   char *ptr;
   
   for(i = 1; i < argc; ++i)
   {
      ptr = argv[i];
      if(*ptr == '-')
      {
         while(*++ptr)
         switch(*ptr)
         {
            case '-':
               ++ptr;
               if(!strcmp(ptr, "help"))
               {
                  puts(help_text);
                  return (0);
               }
               else if(!strcmp(ptr, "version"))
               {
                  puts("wc (butils) "VERSION_STRING);
                  return (0);
               }
            case 'c':
               if(flag_chars)
               {
                  fprintf(stderr, "%s: Can't specify both `-c' and `-m'\n", 
                     argv[0]);
                  fputs("Try `wc --help' for more information.", stderr);
                  return (1);
               }
               flag_bytes = 1;
               break;
            case 'l':
               flag_lines = 1;
               break;
            case 'm':
               if(flag_bytes)
               {
                  fprintf(stderr, "%s: Can't specify both `-c' and `-m'\n",
                     argv[0]);
                  fputs("Try `wc --help' for more information.", stderr);
                  return (1);
               }
               flag_chars = 1;
               break;
            case 'w':
               flag_words = 1;
               break;
            default:
         }
      }
      else
      {
         if((in = fopen(argv[i], "r")) == NULL)
         {
            fprintf(stderr, "%s: Could not open `%s' for reading.\n", 
               argv[0], argv[i]);
            fputs("Try `wc --help' for more information.\n", stderr);
            return (1);
         }
         if(flag_chars == 1)
            flag_bytes = 1;
         if(!flag_lines && !flag_words && !flag_bytes)
            flag_lines = flag_words = flag_bytes = 1;
         do_count();
         if(flag_lines)
            printf("%7lu ", lines_count);
         if(flag_words)
            printf("%7lu ", words_count);
         if(flag_bytes)
            printf("%7lu ", bytes_count);
         printf(argv[i]);   
         putchar('\n');
         bytes_count = 0;
         words_count = 0;
         lines_count = 0;
      }
   }
   return (0);
}